/*

Copyright 2011 Andreas Massouras

This file is part of Primer-Initiated Sequence Synthesis (PrInSeS)

For license information, please refer to the LICENSE document included in the distribution.
By downloading or using this software, you accept and are bound by the terms of the license.

*/

#ifndef SAM_H
#define SAM_H

#include "Util.h"
#include "Vector.h"
#include "Genome.h"

extern "C" {
#include "sam.h"
};


class SamPileup
{
public:
	// minDepth = minimum read depth for calling a SNP
	SamPileup(const char* filename, const Genome&, uint16_t minDepth = 0, uint8_t minQuality = 20);

	int32_t depthAtPosition(GenomicPosition pos) const { return _genomeReadDepth[pos]; }
	double averageReadDepth(GenomicPosition pos, GenomicPosition len, GenomicPosition flankingLen = 0) const;

	double averageReadDepth() const { return _genomeReadDepth.mean(); }
	GenomicPosition snpCount() const { return _snpCount; }

	const Genome& targetGenome() const { return _targetGenome; }
	const Genome& referenceGenome() const { return _referenceGenome; }

	void outputDepthStats(ostream&) const;

private:
	static int pileupCallback(uint32_t, uint32_t, int, const bam_pileup1_t *, void *);
	void pileupCallback(uint32_t chromoId, uint32_t chromoPos, int readDepth, const bam_pileup1_t*);

	const Genome& _referenceGenome;
	VectorI _samChromosomeMap;
	VectorI _genomeReadDepth;
	Genome _targetGenome;	// with called SNPs
	GenomicPosition _snpCount;
	uint16_t _minDepth;
	uint8_t _minQuality;	// minimum mapping quality as defined in the SAM specification
};


#endif // SAM_H
